<?php

namespace Drupal\entity_bundle_scaffold\Commands;

use Consolidation\AnnotatedCommand\CommandData;
use Consolidation\SiteAlias\SiteAliasManagerAwareInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\entity_bundle_scaffold\Service\Generator\EntityBundleClassGenerator;
use Drush\Commands\DrushCommands;
use PhpCsFixer\Console\Application as PhpCsFixerApplication;
use PhpParser\PrettyPrinter\Standard as PrettyPrinter;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Filesystem\Filesystem;

/**
 * Drush hooks for the entity:bundle-class-generate command.
 */
class EntityBundleClassHooks extends DrushCommands implements SiteAliasManagerAwareInterface {

  use RunCommandTrait;

  /**
   * The entity field manager.
   *
   * @var \Drupal\Core\Entity\EntityFieldManager
   */
  protected $entityFieldManager;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The entity bundle class generator.
   *
   * @var \Drupal\entity_bundle_scaffold\Service\Generator\EntityBundleClassGenerator
   */
  protected $entityBundleClassGenerator;

  /**
   * The PHP code pretty printer.
   *
   * @var \PhpParser\PrettyPrinter\Standard
   */
  protected $prettyPrinter;

  /**
   * The filesystem service.
   *
   * @var \Symfony\Component\Filesystem\Filesystem
   */
  protected $fileSystem;

  /**
   * Constructs an EntityBundleClassHooks object.
   */
  public function __construct(
    EntityFieldManagerInterface $entityFieldManager,
    ConfigFactoryInterface $configFactory,
    EntityBundleClassGenerator $entityBundleClassGenerator
  ) {
    $this->entityFieldManager = $entityFieldManager;
    $this->configFactory = $configFactory;
    $this->entityBundleClassGenerator = $entityBundleClassGenerator;
    $this->prettyPrinter = new PrettyPrinter();
    $this->fileSystem = new Filesystem();
  }

  /**
   * Add the bundle-class-output-module option to a command.
   *
   * @hook option field:create
   */
  public function hookOptionFieldCreate(Command $command): void {
    $this->addModuleOption($command);
  }

  /**
   * Set the default value for the bundle-class-output-module option.
   *
   * @hook init field:create
   */
  public function hookInitFieldCreate(): void {
    $this->setDefaultValue();
  }

  /**
   * Generate field getters after running the field:create command.
   *
   * @hook post-command field:create
   */
  public function hookPostFieldCreate($result, CommandData $commandData): void {
    $module = $commandData->input()->getOption('bundle-class-output-module');
    if ($module === NULL) {
      return;
    }

    $entityType = $commandData->input()->getArgument('entityType');
    $bundle = $commandData->input()->getArgument('bundle');
    $fieldName = $commandData->input()->getOption('field-name');
    $field = $this->entityFieldManager->getFieldDefinitions($entityType, $bundle)[$fieldName] ?? NULL;

    if (!$field) {
      return;
    }

    if (!$statement = $this->entityBundleClassGenerator->appendFieldGettersToExistingClass($entityType, $bundle, [$field])) {
      return;
    }

    $this->logger()->notice('Adding new getter to entity bundle class...');
    $output = $this->prettyPrinter->prettyPrintFile([$statement]);
    $destination = $this->entityBundleClassGenerator->getEntityBundleClassPath($entityType, $bundle)
      ?? $this->entityBundleClassGenerator->buildEntityBundleClassPath($entityType, $bundle, $module);

    if (!file_exists($destination)) {
      return;
    }

    $this->fileSystem->remove($destination);
    $this->fileSystem->appendToFile($destination, $output);

    if (class_exists(PhpCsFixerApplication::class)) {
      $this->logger()->notice('Formatting entity bundle class...');
      $this->drush('phpcs:fix', [], ['path' => $destination]);
    }

    $this->logger()->success('Successfully updated entity bundle class.');
  }

  /**
   * Add the bundle-class-output-module option to a command.
   *
   * @hook option nodetype:create
   */
  public function hookOptionNodeTypeCreate(Command $command): void {
    $this->addModuleOption($command);
  }

  /**
   * Set the default value for the bundle-class-output-module option.
   *
   * @hook init nodetype:create
   */
  public function hookInitNodeTypeCreate(): void {
    $this->setDefaultValue();
  }

  /**
   * Generate an entity bundle class for a node type.
   *
   * @hook post-command nodetype:create
   */
  public function hookPostNodeTypeCreate($result, CommandData $commandData): void {
    $this->generateClass($commandData, 'node');
  }

  /**
   * Add the bundle-class-output-module option to a command.
   *
   * @hook option vocabulary:create
   */
  public function hookOptionVocabularyCreate(Command $command): void {
    $this->addModuleOption($command);
  }

  /**
   * Set the default value for the bundle-class-output-module option.
   *
   * @hook init vocabulary:create
   */
  public function hookInitVocabularyCreate(): void {
    $this->setDefaultValue();
  }

  /**
   * Generate an entity bundle class for a taxonomy vocabulary.
   *
   * @hook post-command vocabulary:create
   */
  public function hookPostVocabularyCreate($result, CommandData $commandData): void {
    $this->generateClass($commandData, 'taxonomy_term');
  }

  /**
   * Add the bundle-class-output-module option to a command.
   *
   * @hook option eck:bundle:create
   */
  public function hookOptionEckBundleCreate(Command $command): void {
    $this->addModuleOption($command);
  }

  /**
   * Set the default value for the bundle-class-output-module option.
   *
   * @hook init eck:bundle:create
   */
  public function hookInitEckBundleCreate(): void {
    $this->setDefaultValue();
  }

  /**
   * Generate an entity bundle class for an ECK bundle.
   *
   * @hook post-command eck:bundle:create
   */
  public function hookPostEckBundleCreate($result, CommandData $commandData): void {
    $this->generateClass($commandData, $commandData->input()->getArgument('entityType'));
  }

  /**
   * Add the bundle-class-output-module option to a command.
   *
   * @hook option paragraphs:type:create
   */
  public function hookOptionParagraphsTypeCreate(Command $command): void {
    $this->addModuleOption($command);
  }

  /**
   * Generate an entity bundle class for a paragraph type.
   *
   * @hook init paragraphs:type:create
   */
  public function hookInitParagraphsTypeCreate(): void {
    $this->setDefaultValue();
  }

  /**
   * Generate an entity bundle class for a paragraph type.
   *
   * @hook post-command paragraphs:type:create
   */
  public function hookPostParagraphsTypeCreate($result, CommandData $commandData): void {
    $this->generateClass($commandData, 'paragraph');
  }

  /**
   * Add the bundle-class-output-module option to a command.
   */
  protected function addModuleOption(Command $command): void {
    if ($command->getDefinition()->hasOption('bundle-class-output-module')) {
      return;
    }

    $command->addOption(
      'bundle-class-output-module',
      '',
      InputOption::VALUE_OPTIONAL,
      'The name of the module containing the entity bundle class.'
    );
  }

  /**
   * Set the default value for the bundle-class-output-module option.
   */
  protected function setDefaultValue(): void {
    $module = $this->input->getOption('bundle-class-output-module');

    if (!$module) {
      $default = $this->configFactory
        ->get('entity_bundle_scaffold.settings')
        ->get('generators.bundle_class.output_module');

      $this->input->setOption('bundle-class-output-module', $default);
    }
  }

  /**
   * Generate an entity bundle class.
   */
  protected function generateClass(CommandData $commandData, string $entityType): void {
    $bundle = $commandData->input()->getOption('machine-name');
    $module = $commandData->input()->getOption('bundle-class-output-module');

    if (!$module) {
      return;
    }

    $this->drush(
      'entity:bundle-class-generate',
      [
        'show-machine-names' => $commandData->input()->getOption('show-machine-names'),
        'output-module' => $module,
      ],
      ['entityType' => $entityType, 'bundle' => $bundle]
    );
  }

}
