<?php

namespace Drupal\entity_bundle_scaffold\Commands;

use Consolidation\AnnotatedCommand\CommandData;
use Consolidation\SiteAlias\SiteAliasManagerAwareInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\entity_bundle_scaffold\Service\Generator\ControllerClassGenerator;
use Drush\Commands\DrushCommands;
use PhpParser\PrettyPrinter\Standard as PrettyPrinter;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputOption;

/**
 * Drush hooks for integrating the wmcontroller module with other commands.
 */
class WmControllerHooks extends DrushCommands implements SiteAliasManagerAwareInterface {

  use RunCommandTrait;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The entity bundle controller class generator.
   *
   * @var \Drupal\entity_bundle_scaffold\Service\Generator\ControllerClassGenerator
   */
  protected $controllerClassGenerator;

  /**
   * The PHP code pretty printer.
   *
   * @var \PhpParser\PrettyPrinter\Standard
   */
  protected $prettyPrinter;

  /**
   * Constructs a WmControllerHooks object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The config factory.
   * @param \Drupal\entity_bundle_scaffold\Service\Generator\ControllerClassGenerator $controllerClassGenerator
   *   The entity bundle controller class generator.
   */
  public function __construct(
    ConfigFactoryInterface $configFactory,
    ControllerClassGenerator $controllerClassGenerator
  ) {
    $this->configFactory = $configFactory;
    $this->controllerClassGenerator = $controllerClassGenerator;
    $this->prettyPrinter = new PrettyPrinter();
  }

  /**
   * Add the wmcontroller-output-module option to a command.
   *
   * @hook option nodetype:create
   */
  public function hookOptionNodeTypeCreate(Command $command): void {
    $this->addOption($command);
  }

  /**
   * Set the default value for the wmcontroller-output-module option if empty.
   *
   * @hook init nodetype:create
   */
  public function hookInitNodeTypeCreate(): void {
    $this->setDefaultValue();
  }

  /**
   * Generate an entity bundle controller class.
   *
   * @hook post-command nodetype:create
   */
  public function hookPostNodeTypeCreate($result, CommandData $commandData): void {
    $this->generateController($commandData, 'node');
  }

  /**
   * Add the wmcontroller-output-module option to a command.
   *
   * @hook option vocabulary:create
   */
  public function hookOptionVocabularyCreate(Command $command): void {
    $this->addOption($command);
  }

  /**
   * Set the default value for the wmcontroller-output-module option if empty.
   *
   * @hook init vocabulary:create
   */
  public function hookInitVocabularyTypeCreate(): void {
    $this->setDefaultValue();
  }

  /**
   * Generate an entity bundle controller class.
   *
   * @hook post-command vocabulary:create
   */
  public function hookPostVocabularyCreate($result, CommandData $commandData): void {
    $this->generateController($commandData, 'taxonomy_term');
  }

  /**
   * Add the wmcontroller-output-module option to a command.
   */
  protected function addOption(Command $command): void {
    if ($command->getDefinition()->hasOption('wmcontroller-output-module')) {
      return;
    }

    $command->addOption(
      'wmcontroller-output-module',
      '',
      InputOption::VALUE_OPTIONAL,
      'The name of the module containing the wmcontroller class.'
    );
  }

  /**
   * Set the default value for the wmcontroller-output-module option if empty.
   */
  protected function setDefaultValue(): void {
    $module = $this->input->getOption('wmcontroller-output-module');

    if (!$module) {
      $default = $this->configFactory
        ->get('entity_bundle_scaffold.settings')
        ->get('generators.controller.output_module');

      $this->input->setOption('wmcontroller-output-module', $default);
    }
  }

  /**
   * Generate an entity bundle controller class.
   */
  protected function generateController(CommandData $commandData, string $entityType): void {
    $bundle = $commandData->input()->getOption('machine-name');
    $module = $commandData->input()->getOption('wmcontroller-output-module');

    if (!$module) {
      return;
    }

    $this->drush(
      'wmcontroller:generate',
      [
        'show-machine-names' => $commandData->input()->getOption('show-machine-names'),
        'output-module' => $module,
      ],
      ['entityType' => $entityType, 'bundle' => $bundle]
    );
  }

}
