<?php

namespace Drupal\entity_bundle_scaffold\Service\Generator;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ExtensionPathResolver;
use Drupal\Core\File\FileSystemInterface;
use PhpParser\BuilderFactory;
use PhpParser\Node\Stmt;
use PhpParser\ParserFactory;

/**
 * A base class for class generators with helper methods.
 */
abstract class ClassGeneratorBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The entity field manager.
   *
   * @var \Drupal\Core\Entity\EntityFieldManager
   */
  protected $entityFieldManager;

  /**
   * The filesystem service.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * The extension path resolver.
   *
   * @var \Drupal\Core\Extension\ExtensionPathResolver
   */
  protected $extensionPathResolver;

  /**
   * The settings config of this module.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $config;

  /**
   * A factory for PHP parser builders.
   *
   * @var \PhpParser\BuilderFactory
   */
  protected $builderFactory;

  /**
   * A factory for PHP parsers.
   *
   * @var \PhpParser\ParserFactory
   */
  protected $parserFactory;

  /**
   * Constructs a ClassGeneratorBase object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   * @param \Drupal\Core\Entity\EntityFieldManagerInterface $entityFieldManager
   *   The entity field manager.
   * @param \Drupal\Core\File\FileSystemInterface $fileSystem
   *   The filesystem service.
   * @param \Drupal\Core\Extension\ExtensionPathResolver $extensionPathResolver
   *   The extension path resolver.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The config factory.
   */
  public function __construct(
    EntityTypeManagerInterface $entityTypeManager,
    EntityFieldManagerInterface $entityFieldManager,
    FileSystemInterface $fileSystem,
    ExtensionPathResolver $extensionPathResolver,
    ConfigFactoryInterface $configFactory
  ) {
    $this->entityTypeManager = $entityTypeManager;
    $this->entityFieldManager = $entityFieldManager;
    $this->fileSystem = $fileSystem;
    $this->extensionPathResolver = $extensionPathResolver;

    $this->config = $configFactory->get('entity_bundle_scaffold.settings');
    $this->builderFactory = new BuilderFactory();
    $this->parserFactory = new ParserFactory();
  }

  /**
   * Parse a PHP expression into a PhpParser statement object.
   */
  protected function parseExpression(string $expression): Stmt {
    $parser = $this->parserFactory->create(ParserFactory::PREFER_PHP7);
    $statements = $parser->parse('<?php ' . $expression . ';');

    return $statements[0];
  }

  /**
   * Deduplicate and sort use statements.
   */
  protected function cleanUseStatements(Stmt\Namespace_ $namespace): Stmt\Namespace_ {
    $uses = [];

    // Deduplicate.
    foreach ($namespace->stmts as $i => $statement) {
      if (!$statement instanceof Stmt\Use_) {
        continue;
      }

      foreach ($statement->uses as $j => $use) {
        $name = (string) $use->name;
        if (in_array($name, $uses, TRUE)) {
          unset($statement->uses[$j]);
          continue;
        }

        $uses[] = $name;
      }

      if (empty($statement->uses)) {
        unset($namespace->stmts[$i]);
      }
    }

    // Sort.
    usort(
      $namespace->stmts,
      function (Stmt $a, Stmt $b): int {
        if ($a instanceof Stmt\Class_ && $b instanceof Stmt\Use_) {
          return 1;
        }
        return -1;
      }
    );

    return $namespace;
  }

}
